function [Y,X] = tvlsim(G,U,x0,Opt)
% tvlsim     Simulate the response of a time-varying system
%
% [Y,X] = tvlsim(G,U) where G is a TVSS and U is a TVMAT
% [Y,X] = tvlsim(G,U,x0) where x0 is the initial condition


% XXX Simulate on time specified for U.  Check that this is 
% contained within the time horizon on which G is defined.
%
% XXX Add input for ODE options

%% Input Processing
narginchk(2,4);
nin = nargin;
if nin==2
    Nx = order(G);
    x0 = zeros(Nx,1);
    Opt = [];
elseif nin==3
    Opt = [];
end
[A,B,C,D] = ssdata(G);
Tspan = [U.Time(1) U.Time(end)];

%% Simulate System
if isempty(Opt)
    Opt = odeset('RelTol',1e-5,'AbsTol',1e-8);
end
odefh = @(t,x) LOCALderiv(t,x,A,B,U);
[t,x] = ode45( odefh,Tspan,x0,Opt);
X = tvmat(x,t);

%% Construct Outputs
U = evalt(U,t,'tvmat');
C = evalt(C,t,'tvmat');
D = evalt(D,t,'tvmat');
Y = C*X+D*U;

%% 
function xdot = LOCALderiv(t,x,A,B,U)
ut = evalt(U,t);    
At = evalt(A,t);
Bt = evalt(B,t);
xdot = At*x + Bt*ut;

